/**
 * This file is a part of DesQ SNI.
 * DesQ SNI is a Widget to show SN Icons for DesQ Shell.
 * Copyright 2019-2022 Britanicus <marcusbritanicus@gmail.com>
 *
 * This file was originally taken from LXQt project
 * <https://github.com/lxqt/lxqt-panel>
 * Several modifications have been made to suit the needs of DesQ
 * Any and all bug reports are to be filed with DesQ and not LXQt.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * at your option, any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 **/

#pragma once

#include <QObject>
#include <QByteArray>
#include <QList>
#include <QMap>
#include <QString>
#include <QStringList>
#include <QVariant>
#include <QtDBus>

#include "SNITypes.hpp"

/*
 * Proxy class for interface org.kde.StatusNotifierItem
 */
class StatusNotifierItem : public QDBusAbstractInterface {
    Q_OBJECT;

    public:
        static inline const char *staticInterfaceName() {
            return "org.kde.StatusNotifierItem";
        }

        StatusNotifierItem( QString service, QString path, QDBusConnection connection, QObject *parent = nullptr );

        ~StatusNotifierItem();

        Q_PROPERTY( QString AttentionIconName READ attentionIconName )
        QString attentionIconName() const;

        Q_PROPERTY( IconPixmapList AttentionIconPixmap READ attentionIconPixmap )
        IconPixmapList attentionIconPixmap() const;

        Q_PROPERTY( QString AttentionMovieName READ attentionMovieName )
        QString attentionMovieName() const;

        Q_PROPERTY( QString Category READ category )
        QString category() const;

        Q_PROPERTY( QString IconName READ iconName )
        QString iconName() const;

        Q_PROPERTY( IconPixmapList IconPixmap READ iconPixmap )
        IconPixmapList iconPixmap() const;

        Q_PROPERTY( QString IconThemePath READ iconThemePath )
        QString iconThemePath() const;

        Q_PROPERTY( QString Id READ id )
        QString id() const;

        Q_PROPERTY( bool ItemIsMenu READ itemIsMenu )
        bool itemIsMenu() const;

        Q_PROPERTY( QDBusObjectPath Menu READ menu )
        QDBusObjectPath menu() const;

        Q_PROPERTY( QString OverlayIconName READ overlayIconName )
        QString overlayIconName() const;

        Q_PROPERTY( IconPixmapList OverlayIconPixmap READ overlayIconPixmap )
        IconPixmapList overlayIconPixmap() const;

        Q_PROPERTY( QString Status READ status )
        QString status() const;

        Q_PROPERTY( QString Title READ title )
        QString title() const;

        Q_PROPERTY( ToolTip ToolTip READ toolTip )
        ToolTip toolTip() const;

        Q_PROPERTY( int WindowId READ windowId )
        int windowId() const;

    public Q_SLOTS: // METHODS
        QDBusPendingReply<> Activate( int x, int y );
        QDBusPendingReply<> ContextMenu( int x, int y );
        QDBusPendingReply<> Scroll( int delta, const QString& orientation );
        QDBusPendingReply<> SecondaryActivate( int x, int y );

    Q_SIGNALS: // SIGNALS
        void NewAttentionIcon();
        void NewIcon();
        void NewOverlayIcon();
        void NewStatus( const QString& status );
        void NewTitle();
        void NewToolTip();
};
